'''
# Nets Concept Core
NETS - Internet Security Lib.
Use this library only for test and education.

### Modules:
    hack - hacking core.
    crypto - cryptography core.
    secure - security core.

### Functions:
    GrayHackApi, WhiteHackApi, HashCryptoApi, RandCryptoApi, DatsSecureApi

### !Attention!
The author is not responsible for the user's actions, the library is intended for white hacking, Internet security, tests, education and cryptography.
'''

from .hack import *
from .crypto import *
from .secure import *
from .utils import validate_password_strength, logging